/****************************************************************************************************************************************************
  CC3000.c
  Driver for CC3000 module.
****************************************************************************************************************************************************/

// Include Files
#include "CC3000.h"
#include <built_in.h>
#include "stdint.h"
#include "__Net_Wireless_CC3000.h"
#include "Common.h"
#include "Buffers.h"
#include "UART1.h"
#include "__Time.h"
#include "LED.h"
#include "Timers.h"

// Defines
#define SOCKET_INACTIVE  -1
#define DATA_BUFFER_SIZE 1400

// WiFi connections
sfr sbit Net_Wireless_CC3000_CS  at GPIO_PORTB_AHB_DATA.B4;
sfr sbit Net_Wireless_CC3000_Int at GPIO_PORTB_AHB_DATA.B2;
sfr sbit Net_Wireless_CC3000_En  at GPIO_PORTF_AHB_DATA.B5;

sfr sbit Net_Wireless_CC3000_CS_Direction  at GPIO_PORTB_AHB_DIR.B4;
sfr sbit Net_Wireless_CC3000_Int_Direction at GPIO_PORTB_AHB_DIR.B2;
sfr sbit Net_Wireless_CC3000_En_Direction  at GPIO_PORTF_AHB_DIR.B5;



// Constants
const int8_t  WIFI_SUCCESS =  0;
const int8_t    WIFI_ERROR = -1;
const uint16_t   localPort = 1234;

// Variables
// Remove const from ssid and password to prevent a "Suspicious pointer conversion" message 4 times!
// Insert your own ssid and key below ***********
uint8_t ssid[32] = "Hitron";                              // SSID of access point.
uint8_t key[]    = "NewGen7of9";                          // Password for WPA2 encryption.

// Alternate ssid and password
uint8_t AltSSID[6]  = {0x48,0x69,0x74,0x72,0x6F,0x6E};                      // Hitron
uint8_t AltKey[10]  = {0x4E,0x65,0x77,0x47,0x65,0x6E,0x37,0x6F,0x66,0x39};  // NewGen7of9
uint8_t AltBSSID[6] = {0x00,0x00,0x00,0x00,0x00,0x00};                      // Not required

uint8_t  myMacAddr[6] = {0x00,0x14,0xA5,0x76,0x19,0x3f};  // my MAC address
uint32_t  myIpAddr[4] = {0,0,0,0};                        // my IP address
uint32_t  gwIpAddr[4] = {0,0,0,0};                        // gateway (router) IP address
uint32_t    ipMask[4] = {0,0,0,0};                        // network mask (for example : 255.255.255.0)
uint32_t dnsIpAddr[4] = {0,0,0,0};                        // DNS server IP address

uint8_t  timeServer[] = "time-a.nist.gov";                // Time server URL
//uint32_t remoteIpAddr[4]  = {0,0,0,0};
uint8_t  remoteIpAddr[4] = {0,0,0,0};                     // IP address will be acquired by DNS
int16_t  remotePort = 37;                                 // Port the time server listens on

int32_t clientSock;                                       // Socket for the TCP client

uint32_t connectedFlag, dhcpCompleteFlag;                 // Flags
uint32_t finReceive, sockOptValue;

uint32_t timeOutCnt;
tSockAddr localSockAddr, remoteSockAddr;                  // Structures


tNetappIpconfigRetArgs MyConfiguration;                   // Structure for testing GetIpConfig function

uint8_t dataBuff[DATA_BUFFER_SIZE];                       // Buffer for receiving TCP data
uint8_t State = 0;                                        // Start State machine in State 0

uint32_t timeServIpAddr[1];                               // To overcome bad code in CC3000 library

int32_t RetVal;                                           // Global return value for CC3000 library functions
int32_t numOfRcvBytes;                                    // How many bytes were received by TCP
long epoch;                                               // For calculating date/time
TimeStruct my_time;                                       // Structure for date/time
uint8_t ObtainedTime = FALSE;

uint8_t TestIP_Addr[4];                                   // Bad programming in library
uint16_t TestPort[2];                                     // Bad programming in library
tSockAddr TestSocketAddress;
tScanResult TestScan;

uint8_t WiFiTimer;
uint8_t NVMEM[68];

uint32_t Counter;

extern char TxIn;                                         // In UART1.c
extern char TxOut;                                        // In UART1.c



/****************************************************************************************************************************************************
* Diagnostic to display the DHCP assigned values.
****************************************************************************************************************************************************/
void DispDHCP(void){
     char *BufNum;

     // This does not work! Why?
     void Net_Wireless_CC3000_GetIpConfig(tNetappIpconfigRetArgs MyConfiguration);

     BufNum = GetBuffer();
     sprintf(BufNum,"aucIP:              %d.%d.%d.%d\r\n",MyConfiguration.aucIP[0],MyConfiguration.aucIP[1],
                                                          MyConfiguration.aucIP[2],MyConfiguration.aucIP[3]);
     UART1_PutQueue(BufNum);

     BufNum = GetBuffer();
     sprintf(BufNum,"aucSubnetMask:      %d.%d.%d.%d\r\n",MyConfiguration.aucSubnetMask[0],MyConfiguration.aucSubnetMask[1],
                                                          MyConfiguration.aucSubnetMask[2],MyConfiguration.aucSubnetMask[3]);
     UART1_PutQueue(BufNum);

     BufNum = GetBuffer();
     sprintf(BufNum,"aucDefaultGateway:  %d.%d.%d.%d\r\n",MyConfiguration.aucDefaultGateway[0],MyConfiguration.aucDefaultGateway[1],
                                                          MyConfiguration.aucDefaultGateway[2],MyConfiguration.aucDefaultGateway[3]);
     UART1_PutQueue(BufNum);

     BufNum = GetBuffer();
     sprintf(BufNum,"aucDHCPServer:      %d.%d.%d.%d\r\n",MyConfiguration.aucDHCPServer[0],MyConfiguration.aucDHCPServer[1],
                                                          MyConfiguration.aucDHCPServer[2],MyConfiguration.aucDHCPServer[3]);
     UART1_PutQueue(BufNum);

     BufNum = GetBuffer();
     sprintf(BufNum,"aucDNSServer:       %d.%d.%d.%d\r\n",MyConfiguration.aucDNSServer[0],MyConfiguration.aucDNSServer[1],
                                                          MyConfiguration.aucDNSServer[2],MyConfiguration.aucDNSServer[3]);
     UART1_PutQueue(BufNum);

     BufNum = GetBuffer();
     sprintf(BufNum,"uaMacAddr:          %d.%d.%d.%d\r\n",MyConfiguration.uaMacAddr[0],MyConfiguration.uaMacAddr[1],
                                                          MyConfiguration.uaMacAddr[2],MyConfiguration.uaMacAddr[3],
                                                          MyConfiguration.uaMacAddr[4],MyConfiguration.uaMacAddr[5]);
     UART1_PutQueue(BufNum);

     BufNum = GetBuffer();
     sprintf(BufNum,"uaSSID:             %d.%d.%d.%d\r\n",MyConfiguration.uaSSID[0],MyConfiguration.uaSSID[1],
                                                          MyConfiguration.uaSSID[2],MyConfiguration.uaSSID[3],
                                                          MyConfiguration.uaSSID[4]);
     UART1_PutQueue(BufNum);
}

/****************************************************************************************************************************************************
* Diagnostic to display the local socket IP address and port.
****************************************************************************************************************************************************/
void DispSocket(void) {
     char *BufNum;

     RetVal = Net_Wireless_CC3000_SocketGetInetAddr(TestIP_Addr, TestPort, &localSockAddr);

     BufNum = GetBuffer();
     sprintf(BufNum,"Local socket IP: %d.%d.%d.%d\r\n",TestIP_Addr[0],TestIP_Addr[1],TestIP_Addr[2],TestIP_Addr[3]);
     UART1_PutQueue(BufNum);

     BufNum = GetBuffer();
     sprintf(BufNum,"Local socket Port: %d\r\n",TestPort[0]);
     UART1_PutQueue(BufNum);

     RetVal = Net_Wireless_CC3000_SocketGetInetAddr(TestIP_Addr, TestPort, &remoteSockAddr);

     BufNum = GetBuffer();
     sprintf(BufNum,"Remote socket IP: %d.%d.%d.%d\r\n",TestIP_Addr[0],TestIP_Addr[1],TestIP_Addr[2],TestIP_Addr[3]);
     UART1_PutQueue(BufNum);

     BufNum = GetBuffer();
     sprintf(BufNum,"Remote socket Port: %d\r\n",TestPort[0]);
     UART1_PutQueue(BufNum);
}

/****************************************************************************************************************************************************
* Diagnostic to display the ssid of access points in the area.
****************************************************************************************************************************************************/
void DispSSID(void) {
     char *BufNum;
     char Length;

     RetVal = Net_Wireless_CC3000_GetScanResult(0, &TestScan);

     if (TestScan.status == _NET_WIRELESS_CC3000_SCAN_STATUS_VALID) {
         BufNum = GetBuffer();
         sprintf(BufNum,"\nstatus: %d\r\n",TestScan.status);
         UART1_PutQueue(BufNum);

         BufNum = GetBuffer();
         sprintf(BufNum,"netIndex: %d\r\n",TestScan.netIndex);
         UART1_PutQueue(BufNum);

         BufNum = GetBuffer();
         sprintf(BufNum,"rssi: %d\r\n",TestScan.rssi);
         UART1_PutQueue(BufNum);

         BufNum = GetBuffer();
         sprintf(BufNum,"securityMode: %d\r\n",TestScan.securityMode);
         UART1_PutQueue(BufNum);

         BufNum = GetBuffer();
         sprintf(BufNum,"ssidNameLength: %d\r\n",TestScan.ssidNameLength);
         UART1_PutQueue(BufNum);

         BufNum = GetBuffer();
         UART1_PutQueue("ssidName: ");
         Length = TestScan.ssidNameLength;
         strncpy(BufNum,TestScan.ssidName,Length);
         strcat(BufNum,"\r\n");
         UART1_PutQueue(BufNum);

         BufNum = GetBuffer();
         sprintf(BufNum," bssid: %d:%d:%d:%d:%d:%d\r\n",TestScan.bssid[0],TestScan.bssid[0],TestScan.bssid[0],TestScan.bssid[0],TestScan.bssid[0],TestScan.bssid[0]);
         UART1_PutQueue(BufNum);
     } else {
         UART1_PutQueue("No more APs\r\n");
     }
}

/****************************************************************************************************************************************************
* Diagnostic to display the non-volatile memory files.
****************************************************************************************************************************************************/
void DispNVMEM(void) {
     char *BufNum;

     RetVal = Net_Wireless_CC3000_NvmemRead(_NET_WIRELESS_CC3000_NVMEM_MAC_FILEID, 16, 0, NVMEM);
     BufNum = GetBuffer();
     sprintf(BufNum,"MAC: %02X:%02X:%02X:%02X:%02X:%02X\r\n",NVMEM[0],NVMEM[1],NVMEM[2],NVMEM[3],NVMEM[4],NVMEM[5]);
     UART1_PutQueue(BufNum);

     RetVal = Net_Wireless_CC3000_NvmemRead(_NET_WIRELESS_CC3000_NVMEM_IP_CONFIG_FILEID, 64, 0, NVMEM);
     BufNum = GetBuffer();
     sprintf(BufNum,"IP: %d.%d.%d.%d\r\n",NVMEM[0],NVMEM[1],NVMEM[2],NVMEM[3]);
     UART1_PutQueue(BufNum);

     BufNum = GetBuffer();
     sprintf(BufNum,"IP: %d.%d.%d.%d\r\n",NVMEM[4],NVMEM[5],NVMEM[6],NVMEM[7]);
     UART1_PutQueue(BufNum);

     RetVal = Net_Wireless_CC3000_NvmemRead(_NET_WIRELESS_CC3000_NVMEM_AES128_KEY_FILEID, 16, 0, NVMEM);
     BufNum = GetBuffer();
     sprintf(BufNum,"AES128 key: %02X %02X %02X %02X %02X %02X %02X %02X\r\n",NVMEM[0],NVMEM[1],NVMEM[2],NVMEM[3],NVMEM[4],NVMEM[5],NVMEM[6],NVMEM[7]);
     UART1_PutQueue(BufNum);

     BufNum = GetBuffer();
     sprintf(BufNum,"AES128 key: %02X %02X %02X %02X %02X %02X %02X %02X\r\n",NVMEM[8],NVMEM[9],NVMEM[10],NVMEM[11],NVMEM[12],NVMEM[13],NVMEM[14],NVMEM[15]);
     UART1_PutQueue(BufNum);
}

/****************************************************************************************************************************************************
* Diagnostic to display the CC3000 version.
****************************************************************************************************************************************************/
void DispVer(void) {
     char *BufNum;
     char Ver[2];

     BufNum = GetBuffer();
     if (Net_Wireless_CC3000_NvmemReadSpVer(&Ver) == 0)
         sprintf(BufNum,"Version: %d.%d\r\n",Ver[0],Ver[1]);
     else
         strcpy(BufNum,"Error\r\n");

     UART1_PutQueue(BufNum);
}


/***************************************************************************************************************************************************/




void SSI1_Interrupt() iv IVT_INT_SSI1 ics ICS_AUTO {
     SSI_ICR_EOTIC_SSI1_ICR_bit = 1;      // Clear EOTIC interrupt
     LED_Set_Green();
}

/****************************************************************************************************************************************************
  Initialize SPI1 for the CC3000 chip.
****************************************************************************************************************************************************/
void SPI1_Initialize(void) {
     SPI1_Init_Advanced(15000000, _SPI_MASTER, _SPI_8_BIT | _SPI_CLK_IDLE_LOW | _SPI_SECOND_CLK_EDGE_TRANSITION, &_GPIO_MODULE_SPI1_B5E45_AHB);
/*
     SYSCTL_RCGCSSI_R1_bit = 1;                  // Enable clock to SPI1
     SYSCTL_RCGCGPIO_R1_bit = 1;                 // Enable clock to Port B
     SYSCTL_RCGCGPIO_R4_bit = 1;                 // Enable clock to Port E

     GPIO_PORTE_AHB_AFSEL4_bit = 1;              // Alternate function for PE4 (SPI1 MOSI)
     GPIO_PORTE_AHB_AFSEL5_bit = 1;              // Alternate function for PE5 (SPI1 MISO)
     GPIO_PORTB_AHB_AFSEL5_bit = 1;              // Alternate function for PB5 (SPI1 SCLK)

     GPIO_PORTE_AHB_PCTL |= 0x00FF0000;          // SPI1RX on PE5, SPI1TX on PE4
     GPIO_PORTB_AHB_PCTL |= 0x00F00000;          // SPI1CLK on PB5

     GPIO_PORTE_AHB_DEN4_bit = 1;                // Digital enable PE4
     GPIO_PORTE_AHB_DEN5_bit = 1;                // Digital enable PE5
     GPIO_PORTB_AHB_DEN5_bit = 1;                // Digital enable PB5

     GPIO_PORTE_AHB_PUR |= 0x20;                 // Pull-up on PE5 (SPI1 MISO)

     SSI_CR1_SSE_SSI1_CR1_bit = 0;               // Make sure SSI1 is disable
     SSI1_CR0 = 0x0387;                          // SCR = 3, SPH = 1, SPO = 0, DSS = 7 (8-bit data)
     SSI1_CR1 = 0x00000000;                      // HSCLKEN has to be done after SSE bit is set in CR1
     SSI1_CPSR = 0x00000002;                     // CPSDVSR = 2;

     //SSI1_PP = 0x01;                           // Read only!!!!
     SSI1_CC = 0x00000000;                       // Normal system clock

     SSI_CR1_SSE_SSI1_CR1_bit = 1;               // Enable SSI1
     SSI_CR1_HSCLKEN_SSI1_CR1_bit = 1;           // Set HSCLKEN
*/
     //SSI_IM_EOTIM_SSI1_IM_bit =1;                // Allow EOT interrupts
}




/****************************************************************************************************************************************************
  External interrupt - GPIOB interrupt on PB2
  Used by CC3000 chip
****************************************************************************************************************************************************/
void GPIOInterrupt() iv IVT_INT_GPIOB ics ICS_AUTO {
     Net_Wireless_CC3000_IntGpioHandler();
}

/****************************************************************************************************************************************************
  Timer0A interrupt

****************************************************************************************************************************************************/
void Timer0A_interrupt() iv IVT_INT_TIMER0A_16_32_bit {
     TIMER_ICR_TATOCINT_bit = 1;
     timeOutCnt++;
     Net_Wireless_CC3000_Time++;
}

/****************************************************************************************************************************************************
  Function InitTimer0A()

****************************************************************************************************************************************************/
void InitTimer0A(void) {
     SYSCTL_RCGCTIMER_R0_bit = 1;
     EnableInterrupts();
     TIMER_CTL_TAEN_bit = 0;           // Disable timer0
     TIMER0_CFG   = 4;
     TIMER0_TAPR  = 1;
     TIMER0_TAMR |= 2;                 // Capture on edge
     TIMER0_TAILR = 60000;             // Timer0 interval
     NVIC_IntEnable(IVT_INT_TIMER0A_16_32_bit);
     TIMER_IMR_TATOIM_bit = 1;         // Enable time-out interrupts
     TIMER_CTL_TAEN_bit = 1;           // Enable timer0
}

/****************************************************************************************************************************************************
  Function Net_Wireless_CC3000_WaitHandler()
****************************************************************************************************************************************************/
int32_t Net_Wireless_CC3000_WaitHandler() {

     if(Net_Wireless_CC3000_Time > 30000) {  // Check if timeout.
          return -1;  // No response from CC3000 module.
     }
     return 0;
}

/******************************************************************************************************************************************
  Call incoming unsolicited event handler, check wlan status
******************************************************************************************************************************************/
void WiFi_Polling(void) {
     int32_t status;

     Net_Wireless_CC3000_unsolEventHandler();

     status = Net_Wireless_CC3000_StatusGet();
     if (status == _NET_WIRELESS_CC3000_STATUS_DISCONNECTED) {
         Net_Wireless_CC3000_DiconnectFromAP();
         connectedFlag = 0;
         dhcpCompleteFlag = 0;
         UART1_PutQueue("CC3000 polling stopped\r\n");
     }

}

/****************************************************************************************************************************************************
  Function Net_Wireless_CC3000_UsynchCallback()
  This function handles asynchronous events that come from the CC3000 device.
****************************************************************************************************************************************************/

void Net_Wireless_CC3000_UsynchCallback(int32_t eventType, uint8_t *dat, uint8_t length) {
     char *BufNum;

#if DEBUG
     /* Diagnostic */
     BufNum = GetBuffer();
     sprintf(BufNum,"Event type: %04X  Length: %d\r\n",eventType,length);
     UART1_PutQueue(BufNum);
#endif
     switch (eventType) {

         case _NET_WIRELESS_CC3000_EVNT_UNSOL_CONNECT:
              connectedFlag = 1;  // Device connected to AP.
              //UART1_PutQueue("_NET_WIRELESS_CC3000_EVNT_UNSOL_CONNECT\r\n");
              DEBUGOUT("_NET_WIRELESS_CC3000_EVNT_UNSOL_CONNECT\r\n");
              break;

         case _NET_WIRELESS_CC3000_EVNT_UNSOL_DISCONNECT:
              connectedFlag = 0; // Device disconnected from AP.
              dhcpCompleteFlag = 0;
              //UART1_PutQueue("_NET_WIRELESS_CC3000_EVNT_UNSOL_DISCONNECT\r\n");
              DEBUGOUT("_NET_WIRELESS_CC3000_EVNT_UNSOL_DISCONNECT\r\n");
              break;

         case _NET_WIRELESS_CC3000_EVNT_UNSOL_INIT:
              //UART1_PutQueue("_NET_WIRELESS_CC3000_EVNT_UNSOL_INIT\r\n");
              DEBUGOUT("_NET_WIRELESS_CC3000_EVNT_UNSOL_INIT\r\n");
              break;

         case _NET_WIRELESS_CC3000_EVNT_TX_COMPLETE:
              //UART1_PutQueue("_NET_WIRELESS_CC3000_EVNT_TX_COMPLETE\r\n");
              DEBUGOUT("_NET_WIRELESS_CC3000_EVNT_TX_COMPLETE\r\n");
              break;

         case _NET_WIRELESS_CC3000_EVNT_UNSOL_DHCP:
              /*
              if (*(dat + 20) == 0) {
                    UART1_PutQueue("dhcpCompleteFlag is 1\r\n");
                    dhcpCompleteFlag = 1;
              } else {
                    UART1_PutQueue("dhcpCompleteFlag is 0\r\n");
                    dhcpCompleteFlag = 0;
              }
              */
              dhcpCompleteFlag = 1;
              //UART1_PutQueue("_NET_WIRELESS_CC3000_EVNT_UNSOL_DHCP\r\n");
              DEBUGOUT("_NET_WIRELESS_CC3000_EVNT_UNSOL_DHCP\r\n");
              break;

         case _NET_WIRELESS_CC3000_EVNT_ASYNC_PING:
              //UART1_PutQueue("_NET_WIRELESS_CC3000_EVNT_ASYNC_PING\r\n");
              DEBUGOUT("_NET_WIRELESS_CC3000_EVNT_ASYNC_PING\r\n");
              break;

         case _NET_WIRELESS_CC3000_EVNT_ASYNC_SIMPLE_CFG_DONE:
              //UART1_PutQueue("_NET_WIRELESS_CC3000_EVNT_ASYNC_SIMPLE_CFG_DONE\r\n");
              DEBUGOUT("_NET_WIRELESS_CC3000_EVNT_ASYNC_SIMPLE_CFG_DONE\r\n");
              break;

         case _NET_WIRELESS_CC3000_EVNT_KEEPALIVE:
              //UART1_PutQueue("_NET_WIRELESS_CC3000_EVNT_KEEPALIVE\r\n");
              DEBUGOUT("_NET_WIRELESS_CC3000_EVNT_KEEPALIVE\r\n");
              break;

         case _NET_WIRELESS_CC3000_EVNT_BSD_TCP_CLOSE_WAIT:
              if (dat[0] == clientSock) {  // data[0] represents the socket id, for which FIN was received.
                  finReceive = 1;
              }
              //UART1_PutQueue("_NET_WIRELESS_CC3000_EVNT_BSD_TCP_CLOSE_WAIT\r\n");
              DEBUGOUT("_NET_WIRELESS_CC3000_EVNT_BSD_TCP_CLOSE_WAIT\r\n");
              break;

         case _NET_WIRELESS_CC3000_EVENT_CAN_SHUT_DOWN:
              //UART1_PutQueue("_NET_WIRELESS_CC3000_EVENT_CAN_SHUT_DOWN\r\n");
              DEBUGOUT("_NET_WIRELESS_CC3000_EVENT_CAN_SHUT_DOWN\r\n");
              break;
     }
}


/****************************************************************************************************************************************************
  This is the state machine version
****************************************************************************************************************************************************/
void WiFi_StateMachine(void) {
     uint32_t timeOutDHCP = 14400;
     uint32_t timeOutARP = 3600;
     uint32_t timeOutKeepalive = 20;
     uint32_t timeOutInactivity = 20;
     char *BufNum;



     switch (State) {

         case 0: // Stay in this state until released by user

             break;

     /**********************************************************************************************************************************************/
     // Hardware setup

         case 1: // Initialize CC3000 timer
             //UART1_PutQueue("Initializing WiFi timer\r\n");
             DEBUGOUT("Initializing WiFi timer\r\n");
             InitTimer0A(); // Initialize CC3000 timer
             State = 2;
             break;

         case 2: // Start SPI, initialize CC3000 driver, register asynchronous event handler, enable external interrupt, power up CC3000 module
             UART1_PutQueue("CC3000 configuring\r\n");

             SPI1_Initialize();

             (void) Net_Wireless_CC3000_Init();
             (void) Net_Wireless_CC3000_regAsynHandler(Net_Wireless_CC3000_UsynchCallback);
             NVIC_IntEnable(IVT_INT_GPIOB);

             UART1_PutQueue("CC3000 starting\r\n");
             (void) Net_Wireless_CC3000_Start(0);
             State = 3;
             break;

         case 3: // Set MAC address into CC3000
             RetVal = Net_Wireless_CC3000_SetMacAddr(myMacAddr);
             if (RetVal < 0) {
                 UART1_PutQueue("Set MAC address error\r\n");
                 State = 0;
             } else {
                 UART1_PutQueue("CC3000 MAC address set\r\n");
                 State = 4;
             }
             break;

         case 4: // Set CC3000 IP addresses to 0 to invoke DHCP
             RetVal = Net_Wireless_CC3000_SetIpAddr(myIpAddr, ipMask, gwIpAddr, dnsIpAddr);
             if (RetVal < 0) {
                 UART1_PutQueue("Set IP address error\r\n");
                 State = 0;
             } else {
                 UART1_PutQueue("CC3000 IP addresses set to 0 for DHCP\r\n");
                 State = 5;
             }
             break;

         case 5: // Set connect policy into CC3000
             RetVal = Net_Wireless_CC3000_SetConnectPolicy(0,0,0);
             if (RetVal < 0){
                 UART1_PutQueue("Set connect policy error\r\n");
                 State = 0;
             } else {
                 UART1_PutQueue("CC3000 connect policy set\r\n");
                 State = 6;
             }
             break;

         case 6: // Set timeout parameters into CC3000
             RetVal = Net_Wireless_CC3000_SetTimeoutParams(&timeOutDHCP, &timeOutARP, &timeOutKeepalive, &timeOutInactivity);
             if (RetVal < 0){
                 UART1_PutQueue("Set timeout parameters error\r\n");
                 State = 0;
             } else {
                 UART1_PutQueue("CC3000 timeout parameters set\r\n");
                 State = 7;
             }
             break;

         case 7: // Disconnect CC3000 from access point (don't know if this is necessary)
             RetVal = Net_Wireless_CC3000_DiconnectFromAP();
             UART1_PutQueue("CC3000 disconnected from AP\r\n");
             State = 8;
             break;

         case 8: // Restart CC3000 (don't know if this is necessary)
             (void) Net_Wireless_CC3000_Stop();
             (void) Net_Wireless_CC3000_Start(0);
             UART1_PutQueue("\nCC3000 restarting\r\n");
             InitTimer0A(); // Re-initialize timer
             finReceive = 0;
             UART1_PutQueue("CC3000 connecting to AP. Please wait...\r\n");
             State = 9;
             break;

         case 9: // Wait here for all messages to be transmitted
             if (TxIn != TxOut) {
                 State = 9; // Stay in this state
             } else {
                 State = 10; // Go to next state
             }
             break;

         case 10: // Connect to access point using ssid name and security key
             RetVal = Net_Wireless_CC3000_ConnectToAP(_NET_WIRELESS_CC3000_SEC_WPA2, ssid, strlen(ssid), 0, key, strlen(key));

             // Alternate ssid and key
             //RetVal = Net_Wireless_CC3000_ConnectToAP(_NET_WIRELESS_CC3000_SEC_WPA2, AltSSID, sizeof(AltSSID), 0, AltKey, sizeof(AltKey));

             if (RetVal < 0){
                 UART1_PutQueue("Connect to AP error\r\n");
                 State = 0;
             } else {
                 UART1_PutQueue("CC3000 connected to AP\r\n");
                 State = 11;
             }
             break;

         case 11: // Waiting for unsolicited connect event
             if (connectedFlag != 1) {
                 State = 11; // Stay in this state
             } else {
                   //UART1_PutQueue("Unsolicited connect event\r\n");
                   State = 12; // Go to next state
             }
             break;

         case 12: // Waiting for unsolicited DHCP event
             if (dhcpCompleteFlag != 1) {
                 State = 12; // Stay in this state
             } else {
                   //UART1_PutQueue("Unsolicited DHCP event\r\n");
                   //DispDHCP(); // Try to display the assigned DHCP addresses. Doesn't work!
                   State = 13; // Go to next state
             }
             break;

     /**********************************************************************************************************************************************/
     // Get date and time from server.

         case 13: // Get a TCP socket
             clientSock = Net_Wireless_CC3000_SocketCreate(_NET_WIRELESS_CC3000_SOCK_DOMAIN_AF_INET, _NET_WIRELESS_CC3000_SOCK_TYPE_STREAM,
                                                   _NET_WIRELESS_CC3000_SOCK_PROTO_TCP);

             if(clientSock == _NET_WIRELESS_CC3000_ERROR)
             {
                 UART1_PutQueue("Socket error\r\n");
                 State = 0;
             } else {
#if DEBUG
                 UART1_PutQueue("Got socket OK\r\n");
                 BufNum = GetBuffer();
                 sprintf(BufNum,"Socket: %d\r\n",clientSock);  // Display the obtained socket number
                 UART1_PutQueue(BufNum);
#endif
                 State = 14;
             }
             break;

         case 14: // Set receive timeout.
             sockOptValue  = 100;
             RetVal = Net_Wireless_CC3000_SocketSetOptions(clientSock, _NET_WIRELESS_CC3000_SOCK_LEVEL,
                                           _NET_WIRELESS_CC3000_SOCK_OPT_RCV_TIMEOUT, &sockOptValue, sizeof(sockOptValue));
             if (RetVal < 0) {
                 UART1_PutQueue("Set socket options error 1\r\n");
                 State = 0;
             } else {
                 //UART1_PutQueue("Set receive timeout OK\r\n");
                 DEBUGOUT("Set receive timeout OK\r\n");
                 State = 15;
             }
             break;

         case 15: // Receive: non-blocking
             sockOptValue  = _NET_WIRELESS_CC3000_SOCK_OPT_NONBLOCK_OFF;
             RetVal = Net_Wireless_CC3000_SocketSetOptions(clientSock, _NET_WIRELESS_CC3000_SOCK_LEVEL,
                                           _NET_WIRELESS_CC3000_SOCK_OPT_RCV_NONBLOCK, &sockOptValue, sizeof(sockOptValue));
             if (RetVal < 0) {
                 UART1_PutQueue("Set socket options error 2\r\n");
                 State = 0;
             } else {
                 //UART1_PutQueue("Set receive blocking OK\r\n");
                 DEBUGOUT("Set receive blocking OK\r\n");
                 State = 16;
             }
             break;

         case 16: // Fill local socket structure
             (void) Net_Wireless_CC3000_SocketSetInetAddr(0, localPort, &localSockAddr);
             //UART1_PutQueue("Local socket structure set\r\n");
             DEBUGOUT("Local socket structure set\r\n");
             State = 17;
             break;

         case 17: // Bind TCP socket
             RetVal = Net_Wireless_CC3000_SocketBind(clientSock, &localSockAddr, sizeof(localSockAddr));
             if (RetVal < 0) {
                 UART1_PutQueue("Socket bind error 1\r\n");
                 State = 0;
             } else {
                 //UART1_PutQueue("Socket bind OK\r\n");
                 DEBUGOUT("Socket bind OK\r\n");
                 State = 18;
             }
             break;

         case 18: // Get the IP address of a server
             //UART1_PutQueue("Getting host IP address by DNS\r\n");
             DEBUGOUT("Getting host IP address by DNS\r\n");
             RetVal = Net_Wireless_CC3000_SocketGetHostByName(timeServer, strlen(timeServer), timeServIpAddr);
             if (RetVal < 0) {
                 UART1_PutQueue("Get host by name error\r\n");
                 State = 0;
             } else {
                 //UART1_PutQueue("Got host IP address\r\n");
                 DEBUGOUT("Got host IP address\r\n");
                 State = 19;
             }
             break;

         case 19: // Convert server IP address into an array
             remoteIpAddr[0] = (timeServIpAddr[0]);
             remoteIpAddr[1] = (timeServIpAddr[0] >> 8);
             remoteIpAddr[2] = (timeServIpAddr[0] >> 16);
             remoteIpAddr[3] = (timeServIpAddr[0] >> 24);
#if DEBUG
             BufNum = GetBuffer();
             sprintf(BufNum,"Remote IP address: %d.%d.%d.%d\r\n",remoteIpAddr[0],remoteIpAddr[1],remoteIpAddr[2],remoteIpAddr[3]);
             UART1_PutQueue(BufNum);
#endif
             State = 20;
             break;

         case 20: // Fill remote socket structure
             (void) Net_Wireless_CC3000_SocketSetInetAddr(remoteIpAddr, remotePort, &remoteSockAddr);
             //UART1_PutQueue("Remote socket structure set\r\n");
             DEBUGOUT("Remote socket structure set\r\n");
#if DEBUG
             DispSocket();  // Diagnostic
#endif
             State = 21;
             break;

         case 21: // Connect TCP to server
             RetVal = Net_Wireless_CC3000_SocketConnect(clientSock, &remoteSockAddr, sizeof(remoteSockAddr));
             Counter = 0;
             State = 22;
             break;

         case 22: // Hold here until data is received
             if (RetVal != 0) {
                 Counter += 1;
                 State = 22; // Stay here
             } else {
#if DEBUG
                 BufNum = GetBuffer();
                 sprintf(BufNum,"Counter: %d\r\n",Counter);
                 UART1_PutQueue(BufNum);
#endif
                 State = 23;
             }
             break;

         case 23: // Get data from server
             LED_Set_Green(); // Diagnostic LED on
             numOfRcvBytes = 0;
             numOfRcvBytes = Net_Wireless_CC3000_SocketReceive(clientSock, dataBuff, DATA_BUFFER_SIZE, 0);

             if (numOfRcvBytes <= 0) {
                 UART1_PutQueue("No data received\r\n");
                 State = 27; // Go close socket
             } else {
                 //UART1_PutQueue("Data received from server\r\n");
                 DEBUGOUT("Data received from server\r\n");
                 State = 24;
             }
             break;

         case 24: // Diagnostic
             LED_Clr_Green(); // Diagnostic LED off
#if DEBUG
             BufNum = GetBuffer();
             sprintf(BufNum,"Bytes received: %d\r\n",numOfRcvBytes);
             UART1_PutQueue(BufNum);
#endif
             State = 25;
             break;

         case 25: // Fill time structure with received data
             Highest(epoch) = dataBuff[0];
             Higher(epoch)  = dataBuff[1];
             Hi(epoch) = dataBuff[2];
             Lo(epoch) = dataBuff[3];

             epoch = epoch - 2208988800 - 14400;  // Convert from seconds passed from 1.1.1900 to UNIX time. Eastern Daylight Time (GMT -4 hrs)
             (void) Time_epochToDate(epoch, &my_time);
             //UART1_PutQueue("Obtained time\r\n");
             DEBUGOUT("Obtained time\r\n");
             State = 26;
             break;

         case 26: // Diagnostic: display date/time
#if DEBUG
             BufNum = GetBuffer();
             sprintf(BufNum,"Time: %d/%02d/%02d %02d:%02d:%02d\r\n",my_time.yy,my_time.mo,my_time.md,my_time.hh,my_time.mn,my_time.ss);
             UART1_PutQueue(BufNum);
#endif
             ObtainedTime = TRUE;
             State = 27;
             break;

         case 27:
             //DispDHCP();   // Doesn't work!
             State = 28;
             break;

         case 28: // Close the TCP connection
             RetVal = Net_Wireless_CC3000_SocketClose(clientSock);
             if (RetVal < 0) {
                 UART1_PutQueue("Socket close error 1\r\n");
             } else {
                 //UART1_PutQueue("Socket closed OK\r\n");
                 DEBUGOUT("Socket closed OK\r\n");
             }
             State = 29;
             break;

         case 29:
             RetVal = Net_Wireless_CC3000_DiconnectFromAP();
             //UART1_PutQueue("CC3000 disconnected from AP\r\n");
             DEBUGOUT("CC3000 disconnected from AP\r\n");
             State = 32;
             break;

         case 30:

             break;

         case 31:

             break;

         case 32: // Reset CC3000
             (void) Net_Wireless_CC3000_Stop();
             UART1_PutQueue("CC3000 reset\r\n");
             State = 0;
             break;

     /**********************************************************************************************************************************************/
         // Scan for access points
         case 33:
             UART1_PutQueue("Initializing WiFi timer\r\n");
             InitTimer0A(); // Initialize CC3000 timer
             State = 34;
             break;

         case 34:
             UART1_PutQueue("CC3000 configuring\r\n");

             SPI1_Initialize();
             (void) Net_Wireless_CC3000_Init();
             (void) Net_Wireless_CC3000_regAsynHandler(Net_Wireless_CC3000_UsynchCallback);
             NVIC_IntEnable(IVT_INT_GPIOB);

             UART1_PutQueue("CC3000 starting\r\n");
             (void) Net_Wireless_CC3000_Start(0);
             State = 35;
             break;

         case 35:
             WiFiTimer = get_timer();          // Get a timer handle
             set_timer(WiFiTimer,2000);         // Set timer for 2s delay
             RetVal = Net_Wireless_CC3000_SetScanParams(1, 100, 200, 5, 0x1fff, -75, 0, 205, 0);
             State = 36;
             break;

         case 36:
             if (check_timer(WiFiTimer) == 0) {  // If timeout
                 State = 37;
             }
             break;

         case 37:
             DispSSID();
             if (TestScan.status == _NET_WIRELESS_CC3000_SCAN_STATUS_VALID) {
                 set_timer(WiFiTimer,2000);
                 State = 36;
             } else {
                 free_timer(WiFiTimer);  // Release Wi-Fi timer back to the pool
                 State = 32;
             }
             break;

     /**********************************************************************************************************************************************/
         // Display non-volatile memory
         case 38:
             UART1_PutQueue("Initializing WiFi timer\r\n");
             InitTimer0A(); // Initialize CC3000 timer
             State = 39;
             break;

         case 39:
             UART1_PutQueue("CC3000 configuring\r\n");

             SPI1_Initialize();
             (void) Net_Wireless_CC3000_Init();
             (void) Net_Wireless_CC3000_regAsynHandler(Net_Wireless_CC3000_UsynchCallback);
             NVIC_IntEnable(IVT_INT_GPIOB);

             UART1_PutQueue("CC3000 starting\r\n");
             (void) Net_Wireless_CC3000_Start(0);
             State = 40;
             break;

         case 40:
             WiFiTimer = get_timer();          // Get a timer handle
             set_timer(WiFiTimer,500);         // Set timer for 500 ms delay
             State = 41;
             break;

         case 41:
             if (check_timer(WiFiTimer) == 0) {  // If timeout
                 State = 42;
             }
             break;

         case 42:
             DispNVMEM();
             DispVer();
             DispSocket();
             State = 32;
             break;



     }
}


/****************************************************************************************************************************************************
* End of File
****************************************************************************************************************************************************/