/**************************************************************************************************
  Timers.h
**************************************************************************************************/
#ifndef TIMERS_H
#define TIMERS_H




// Function Prototypes
void TimerInit(void);
void TimerCheck(void);




/**************************************************************************************************
File:         Timers.h
Description:  Implements simple timer pool. No timer chaining
Resolution:   Depand on how often irqhandler_timer is called (+0/-0.999 second)

Author: Jari Lahti (jari.lahti@violasystems.com)
Version Info: 22.1.2004 - First version (JaL)
**************************************************************************************************/

//#ifndef TIMERS_H
//#define TIMERS_H


// Defines
#define NUMTIMERS 10

struct timers {
     unsigned long int TimerField;
     unsigned short int Free;
};


/**************************************************************************************************
unction:   init_timerpool
Parameters: none
Return val: void
Desc:       Initialize timer pool
**************************************************************************************************/
void init_timerpool(void);


/**************************************************************************************************
Function:   get_timer
Parameters: none
Return val: unsigned short int - returns handle of free timer
Desc:       Gets free timer from the timer pool
**************************************************************************************************/
unsigned short int get_timer(void);


/**************************************************************************************************
Function:   set_timer
Parameters: unsigned short int nbr - number
            unsigned long int tout - time out
Return val: none
Desc:       Initializes given timer to given time-out
**************************************************************************************************/
void set_timer(unsigned short int nbr, unsigned long int tout);


/**************************************************************************************************
Function:   check_timer
Parameters: unsigned short int nbr - number
Return val: unsigned long int - returns the value of given timer
Desc:       Return the value of given timer
**************************************************************************************************/
unsigned long int check_timer(unsigned short int nbr);


/**************************************************************************************************
Function:   free_timer
Parameters: none
Return val: unsigned short int nbr - number
Desc:       Puts the given timer back to the timer pool
**************************************************************************************************/
void free_timer(unsigned short int nbr);


/**************************************************************************************************
Function:   irqhandler_timer
Parameters: void
Return val: void
Desc:       Call this function on timer interrupt
**************************************************************************************************/
void irqhandler_timer(void);




#endif