/****************************************************************************************************************************************************
 * Project name:
     WiFi.c
 * Generated by:
     mikroC
 * Date of creation
     2014/03/30
 * Time of creation
     9:47:00
 * Test configuration:
     MCU:             TM4C129XNCZAD
     Dev.Board:       mikromedia_5_for_TIVA
                      http://www.mikroe.com/mikromedia/5/tiva/
     Oscillator:      120000000 Hz
     SW:              mikroC PRO for ARM
                      http://www.mikroe.com/mikroc/arm/

 Notes:
 1. This project uses the mikroE WiFi library to access a time server and set the Real-Time-Clock circuits to the current date and time.
 2. The Edge Column is set for 150. The DEBUG definition in Common.h is set to 1 enabling verbose diagnostic messages.
 3. UART1 is used for commands and diagnostic responses. This requires a TIVA shield and a USB cable from the shield connector to the PC.
 4. Your WiFi router must be set for WPA2 encryption. Alter the ssid and key texts in CC3000.c with your own ssid name and password.
 5. You can use command 'm' to scan for ssid names in your vicinity. Command 'o' will start the state machines to get date and time.
 6. Command 'p' will display the RTC calendar registers as they increment every second.
 7. If using MikroElektronika Usart Terminal program for commands and messages, first start the Usart Terminal program then select the COM port
    for the FTDI chip and click Connect. Then you can program the flash with mikroProg. Check mark Append New Line box in Usart Terminal
    and enter a command in the Send edit box. Click Send button.
 8. If using the SerialUtiity program supplied with this project, open it first to reserve the COM port. Then, when you program the flash
    in tiva, mikroProg will connect to the right port. Otherwise, you will get an error message with Debug Fail!

****************************************************************************************************************************************************/


#include "LED.h"
#include "UART1.h"
#include "CC3000.h"
#include "stdint.h"
#include "Timers.h"
#include "RTCC.h"



// Variables
extern uint32_t connectedFlag, dhcpCompleteFlag;





void main() {
     
     TimerInit();           // Initialize timer
     //RTC_Init();            // Initialize RTC for counter mode
     RTC_Initialize();      // Initialize RTC for calendar mode
     LED_Initialize();      // Initialize LEDs
     
     UART1_Initialize();    // Initialize UART1
     UART1_PutQueue("Started CC3000 test program\r\n");

     
     EnableInterrupts();
     
     while (1) {
         UART1_Rx();              // Check for incoming UART messages
         UART1_Tx();              // Check for outgoing UART messages
         TimerCheck();            // Check timer pool
         
         
         if ((connectedFlag == 1) || (dhcpCompleteFlag == 1))
              WiFi_Polling();
     
         WiFi_StateMachine();     // Do WiFi state machine
         RTC_StateMachine();      // Do RTC state machine
     }
}